from configparser import ConfigParser
from pathlib import Path
from pyramid.settings import aslist_cronly
from urllib.parse import urlparse


def aslist(value, flatten=True):
    """ Return a list of strings, separating the input based on newlines
    and, if flatten=True (the default), also split on spaces within
    each line."""
    if value is None or value == '':
        return []

    values = aslist_cronly(value)
    if not flatten:
        return values
    result = []
    for value in values:
        subvalues = value.split()
        result.extend(subvalues)
    return result


class TenantConfigurator(object):
    def __init__(self, request=None, config_file=None, settings=None, defaults=None):
        if not config_file and not settings:
            raise ValueError()

        if config_file:
            here = Path('.').cwd()
            # Parse the ini config file.
            self.settings = ConfigParser({'here': str(here)})
            self.settings.read(config_file)

        else:
            self.settings = settings

        self.defaults = defaults or {}
        self.domain = urlparse(request.host_url).netloc if request else None

    def get(self, parameter, section_name, main=True, empty_accepted=False):
        """Returns configuration value in accordance with the wanted key (domain or tenant).

        Args:
            parameter (str).
            section_name (str).
            main (bool): if no direct section is found, should we return the main/default?
            empty_accepted (bool): is an empty value considered valid?

        Returns:
            str: configuration value.
        """
        sections = self.settings.sections()

        # Direct match: 'tenant:kv56Zf4n'.
        if section_name in sections:
            setting = self.settings[section_name].get(parameter)
            if (empty_accepted and setting == '') or setting:
                return setting

        # List match: 'tenant:kv56Zf4n,b78EZc3A,pl6HJtr4'.
        try:
            conf_type, conf_id = section_name.split(':')
            for section in sections:
                if not section.startswith(f'{conf_type}:'):
                    continue

                section_ids = section[section.find(':') + 1:].split(',')
                if conf_id in section_ids:
                    setting = self.settings[section].get(parameter)
                    if (empty_accepted and setting == '') or setting:
                        return setting

        except ValueError:
            pass

        if not main:
            return None

        # No match: use global configuration.
        setting = self.settings['app:main'].get(parameter)
        if (empty_accepted and setting == '') or setting:
            return setting

        # Parameter isn't present in ini file, use default.
        return self.defaults.get(parameter)

    def get_for_all_tenants(self, parameter):
        """Get the configuration for all tenants. Useful for admins.

        Args:
            parameter (str): wanted key.

        Returns:
            dict (str:str): configuration, each key being a tenant + 'default' .
        """
        output = {
            'default': self.settings['app:main'].get(parameter, ''),
        }

        for section in self.settings.sections():
            if not section.startswith('tenant:'):
                continue

            section_ids = section[section.find(':') + 1:].split(',')
            for section_id in section_ids:
                output[section_id] = self.settings[section].get(parameter, '')

        return output

    def get_for_domain(self, parameter, empty_accepted=False):
        """Get the configuration for one parameter for a given domain.

        Args:
            parameter (str): wanted key.
            empty_accepted (bool): is an empty value acceptable? Useful if the default is a value and we want to
                deactivate this for a given domain (e.g. modules/specific).

        Returns:
            str: configuration for the given domain.
        """
        return self.get(parameter, f'domain:{self.domain}', empty_accepted=empty_accepted)

    def get_for_tenant(self, parameter, tenant, main=True, empty_accepted=False):
        """Get the configuration for one parameter for a given tenant.

        Args:
            parameter (str): wanted key.
            tenant (str): wanted tenant.
            main (bool): if no direct section is found, should we return the main/default?
            empty_accepted (bool): is an empty value acceptable? Useful if the default is a value and we want to
                deactivate this for a given tenant (e.g. modules/specific).

        Returns:
            str: configuration for the given tenant.
        """
        return self.get(parameter, f'tenant:{tenant}', main=main, empty_accepted=empty_accepted)
