"""Provide a docker healthcheck and a status endpoint to Python apps, testing persistence, redis and celery."""

import argparse
import collections
from random import randint

import celery.bin.base
import pkg_resources
import requests
import requests_unixsocket
from redis.exceptions import RedisError
from sqlalchemy.exc import SQLAlchemyError

from parsys_utilities.celery_app import app


def healthcheck():
    """Command-line utility. Check if the docker container is healthy.
    In dev, we expose a port, in prod, we use a socket.

    If the get fails, an exception will be raised and the command return will be != 0.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('--dev', action='store_true')
    args, extras = parser.parse_known_args()

    if args.dev:
        requests.get('http://localhost:80/health/', timeout=0.5)
    else:
        requests_unixsocket.get('http+unix://%2Fsrv%2Fweb.sock/health/', timeout=0.5)


def get_module_version(package_name):
    """[serverVersion] Get the version of the server module.

    Args:
        package_name (str).

    Returns:
        str: app version.
    """
    try:
        version = pkg_resources.require(package_name)[0].version

    except (pkg_resources.DistributionNotFound, IndexError):
        return None
    else:
        return version


def db_status(db_session, caller_model):
    """[persistenceService] Check if the database is accessible.

    Args:
        db_session (sqlalchemy.orm.session.Session): sqlalchemy db session.
        caller_model (sqlalchemy.sql.schema.Table): a table used by the app to test if the db is working.

    Returns:
        bool: is the database working?
    """
    try:
        db_session.query(caller_model).count()

    except SQLAlchemyError:
        return False
    else:
        return True


def celery_status():
    """[queueService] Check if celery is running.

    Returns:
        bool: is celery working?
    """
    try:
        celery_inspect = app.control.inspect()

    except celery.bin.base.Error:
        return False
    else:
        # If no worker is running, stats() returns None.
        return celery_inspect.stats() is not None


def redis_status(session):
    """[keyValueService] Validate redis is running.

    Args:
        session (pyramid_redis_sessions.session.RedisSession).

    Returns:
        bool: is redis working?
    """
    test_value = randint(1, 500)

    try:
        session['usage-test-only'] = test_value
        assert session.get('usage-test-only') == test_value
        del session['usage-test-only']

    except (AssertionError, RedisError):
        return False
    else:
        return True


def status_endpoint(request, caller_package, caller_model):
    """Test the app and all related services.

    Args:
        request (pyramid.request.Request).
        caller_package (str): name of app.
        caller_model (sqlalchemy.sql.schema.Table): a table used by the app to test if the db is working.

    Returns:
        dict: app status information.
    """
    status = collections.OrderedDict()
    status['serverVersion'] = get_module_version(caller_package)
    status['parsysUtilitiesVersion'] = get_module_version('parsys_utilities')
    status['webService'] = True

    # Test if the database is accessible.
    status['persistenceService'] = db_status(request.db_session, caller_model)

    # Test if redis is accessible.
    status['keyValueService'] = redis_status(request.session)

    # Test Celery.
    status['queueService'] = celery_status()

    # If any test is False, return error.
    if not all(status.values()):
        request.response.status_int = 500

    return status
